@@ -8,21 +8,19 @@ these plugins with the py.test command line option
 """
 # pragma: no cover
 
+import subprocess
 import sys
 
 
 def main(args=None):
-    import pytest
 
     if args is None:
         args = sys.argv[1:]
 
-    return pytest.main(args + [
-        '--check-links',
-    ], [
-        'no:python',
-    ])
+    return subprocess.call(
+        [sys.executable, "-m", "pytest", "--check-links", "-p", "no:python"] + args
+    )
 
 
-if __name__ == '__main__':
+if __name__ == "__main__":
     sys.exit(main())
